/* jshint node: true */
'use strict';

var DeployPluginBase = require('ember-cli-deploy-plugin');
var azure = require('azure-storage');
var Promise = require('ember-cli/lib/ext/promise');

var DEFAULT_MANIFEST_SIZE = 10;
var AZURE_TABLE_NAME = 'emberdeploy';
var AZURE_MANIFEST_TAG = 'manifest';

module.exports = {
  name: 'ember-cli-deploy-azure-tables',

  createDeployPlugin: function(options) {
    var DeployPlugin = DeployPluginBase.extend({
      name: options.name,
      
      _createClient: function() {
        var connectionString = this.readConfig("connectionString");
        var storageAccount = this.readConfig("storageAccount");
        var storageAccessKey = this.readConfig("storageAccessKey");

        if(connectionString) {
          return azure.createTableService(connectionString);
        } else if(storageAccount && storageAccessKey) {
          return azure.createTableService(storageAccount, storageAccessKey);
        } else {
          throw new Error("Missing connection string or storage account / access key combination.");
        }
      },

      configure: function(/* context */) {
        this._super.apply(this, arguments);

        if(!pluginConfig.connectionString) {
          ['storageAccount', 'storageAccessKey'].forEach(this.ensureConfigPropertySet.bind(this));
        }
      },

      upload: function(/* context */) {
        var client = this._createClient();

        return new Promise(function(resolve, reject) {
          // create table if not already existent
          client.createTableIfNotExists(AZURE_TABLE_NAME, function(error, result, response) {
            if(!error){
              var query = new azure.TableQuery()
                      .where('PartitionKey eq ?', AZURE_MANIFEST_TAG)
                      .and('RowKey eq ?', key);

              // find the list of uploaded revisions
              client.queryEntities(AZURE_TABLE_NAME, query, null, function(error, result, response) {
                if(!error){
                  // has this key already been uploaded once?
                  if(result.entries.length > 0) {
                    reject("Key already in manifest - revision already uploaded or collided.");
                  } else {
                    var entGen = azure.TableUtilities.entityGenerator;
                    var entity = {};
                    entity["PartitionKey"] = entGen.String(AZURE_MANIFEST_TAG);
                    entity["RowKey"] = entGen.String(key);
                    entity["content"] = entGen.String(value);

                    client.insertEntity(AZURE_TABLE_NAME, entity,  function (error, result, response) {
                      if(!error){
                        resolve(result);
                      } else {
                        reject(error);
                      }
                    });
                  }
                } else {
                  reject(error);
                }
              });
            } else {
              reject(error);
            }
          });
        });
      },
      activate: function(/* context */) {
        var client = this._createClient();

        this.list().then(function(existingEntries) {
          if(existingEntries.indexOf(revisionKey) > -1) {
            return true;
          } else {
            reject("Revision not in manifest");
            return false;
          }
        })
        .then(function() {
          var entGen = azure.TableUtilities.entityGenerator;
          var entity = {};
          entity["PartitionKey"] = entGen.String(AZURE_MANIFEST_TAG);
          entity["RowKey"] = entGen.String(uploadKey);
          entity["content"] = entGen.String(revisionKey);

          client.insertOrReplaceEntity(AZURE_TABLE_NAME, entity,  function (error, result, response) {
            if(!error){
              resolve(result);
            } else {
              reject(error);
            }
          });
        });
      },
      list: function(/* context */) {
        var client = this._createClient();

        return new Promise(function(resolve, reject) {
          // create table if not already existent
          client.createTableIfNotExists(AZURE_TABLE_NAME, function(error, result, response) {
            if(!error){
              var query = new azure.TableQuery()
                      .where('PartitionKey eq ?', AZURE_MANIFEST_TAG);

              // find the list of uploaded revisions
              client.queryEntities(AZURE_TABLE_NAME, query, null, function(error, result, response) {
                if(!error) {
                  var sortedEntries = result.entries;
                  sortedEntries.sort(function(a, b) {
                    return new Date(b["Timestamp"]["_"]).getTime() - new Date(a["Timestamp"]["_"]).getTime();
                  });

                  var entries = sortedEntries.map(function(entry) {
                    return entry["RowKey"]["_"];
                  });

                  resolve(entries);
                } else {
                  reject(error);
                }
              });
            } else {
              reject(error);
            }
          });
        });
      }
    });

    return DeployPlugin;
  }
};
